/*
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "IsIndexInputType.h"

#include "Document.h"
#include "HTMLFormElement.h"
#include "HTMLInputElement.h"
#include <wtf/PassOwnPtr.h>

namespace WebCore {

PassOwnPtr<InputType> IsIndexInputType::create(HTMLInputElement* element)
{
    return adoptPtr(new IsIndexInputType(element));
}

const AtomicString& IsIndexInputType::formControlType() const
{
    return emptyAtom;
}

bool IsIndexInputType::supportsRequired() const
{
    return false;
}

PassRefPtr<HTMLFormElement> IsIndexInputType::formForSubmission() const
{
    RefPtr<HTMLFormElement> form = InputType::formForSubmission();
    if (form)
        return form.release();
    // If there is no form, then create a temporary form just to be used for submission.
    Document* document = element()->document();
    form = HTMLFormElement::create(document);
    form->registerFormElement(element());
    form->setMethod("GET");
    if (!document->baseURL().isEmpty()) {
        // We treat the href property of the <base> element as the form action, as per section 7.5
        // "Queries and Indexes" of the HTML 2.0 spec. <http://www.w3.org/MarkUp/html-spec/html-spec_7.html#SEC7.5>.
        form->setAction(document->baseURL().string());
    }
    return form.release();
}

bool IsIndexInputType::shouldRespectListAttribute()
{
    return false;
}

} // namespace WebCore
