/*
 * Copyright (C) 2007, 2008 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

module window {

    interface [
#if defined(V8_BINDING) && V8_BINDING
        CheckDomainSecurity,
#endif
        DelegatingGetOwnPropertySlot,
        DelegatingPutFunction,
        CustomDeleteProperty,
        CustomGetPropertyNames,
        OmitConstructor
    ] History {
        readonly attribute unsigned long length;

        [DoNotCheckDomainSecurity, CallWith=ScriptExecutionContext] void back();
        [DoNotCheckDomainSecurity, CallWith=ScriptExecutionContext] void forward();
        [DoNotCheckDomainSecurity, CallWith=ScriptExecutionContext] void go(in long distance);
        
        [Custom, EnabledAtRuntime] void pushState(in any data, in DOMString title, in optional DOMString url)
            raises(DOMException);
        [Custom, EnabledAtRuntime] void replaceState(in any data, in DOMString title, in optional DOMString url)
            raises(DOMException);
    };

}
