/*
 * Copyright (C) 2006, 2007, 2009, 2010 Apple Inc. All rights reserve
 * Copyright (C) 2006 Samuel Weinig <sam.weinig@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

module html {

    interface [
        HasOverridingNameGetter
    ] HTMLFrameSetElement : HTMLElement {
        attribute [Reflect] DOMString cols;
        attribute [Reflect] DOMString rows;

#if !defined(LANGUAGE_OBJECTIVE_C) || !LANGUAGE_OBJECTIVE_C
        // Event handler attributes
        attribute [DontEnum, WindowEventListener] EventListener onbeforeunload;
        attribute [DontEnum, WindowEventListener] EventListener onhashchange;
        attribute [DontEnum, WindowEventListener] EventListener onmessage;
        attribute [DontEnum, WindowEventListener] EventListener onoffline;
        attribute [DontEnum, WindowEventListener] EventListener ononline;
        attribute [DontEnum, WindowEventListener] EventListener onpopstate;
        attribute [DontEnum, WindowEventListener] EventListener onresize;
        attribute [DontEnum, WindowEventListener] EventListener onstorage;
        attribute [DontEnum, WindowEventListener] EventListener onunload;

#if defined(ENABLE_ORIENTATION_EVENTS) && ENABLE_ORIENTATION_EVENTS
        attribute [DontEnum] EventListener onorientationchange;
#endif

        // Overrides of Element attributes (with different implementation in bindings).
        attribute [DontEnum, WindowEventListener] EventListener onblur;
        attribute [DontEnum, WindowEventListener] EventListener onerror;
        attribute [DontEnum, WindowEventListener] EventListener onfocus;
        attribute [DontEnum, WindowEventListener] EventListener onload;

        // Not implemented yet.
        // attribute [DontEnum, WindowEventListener] EventListener onafterprint;
        // attribute [DontEnum, WindowEventListener] EventListener onbeforeprint;
        // attribute [DontEnum, WindowEventListener] EventListener onredo;
        // attribute [DontEnum, WindowEventListener] EventListener onundo;
#endif
    };

}
