/*
 * Copyright (C) 2007 Apple Inc. All rights reserved
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

module html {

    interface HTMLMarqueeElement : HTMLElement {
        void start();
        void stop();
        
        attribute [Reflect] DOMString behavior;
        attribute [Reflect] DOMString bgColor;
        attribute [Reflect] DOMString direction;
        attribute [Reflect] DOMString height;
        attribute [Reflect] unsigned long hspace;
        attribute long loop setter raises(DOMException);
        attribute long scrollAmount setter raises(DOMException);
        attribute long scrollDelay setter raises(DOMException);
        attribute [Reflect] boolean trueSpeed;
        attribute [Reflect] unsigned long vspace;
        attribute [Reflect] DOMString width;

        // FIXME: Implement the following event handler attributes
        // https://bugs.webkit.org/show_bug.cgi?id=49788
        // attribute EventListener onbounce;
        // attribute EventListener onfinish;
        // attribute EventListener onstart;
    };
}
