/*
 * Copyright (C) 2004, 2005, 2008 Nikolas Zimmermann <zimmermann@kde.org>
 * Copyright (C) 2004, 2005, 2006, 2010 Rob Buis <buis@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef SVGTests_h
#define SVGTests_h

#if ENABLE(SVG)
#include "SVGAnimatedPropertyMacros.h"
#include "SVGStringList.h"

namespace WebCore {

class Attribute;
class QualifiedName;
class SVGElement;

class SVGTests {
public:
    SVGStringList& requiredFeatures();
    SVGStringList& requiredExtensions();
    SVGStringList& systemLanguage();

    bool hasExtension(const String&) const;
    bool isValid() const;

    bool parseMappedAttribute(Attribute*);
    bool isKnownAttribute(const QualifiedName&);

    bool handleAttributeChange(const SVGElement*, const QualifiedName&);
    void synchronizeProperties(SVGElement*, const QualifiedName&);

protected:
    SVGTests();

private:
    void synchronizeRequiredFeatures(SVGElement*);
    void synchronizeRequiredExtensions(SVGElement*);
    void synchronizeSystemLanguage(SVGElement*);

    SVGSynchronizableAnimatedProperty<SVGStringList> m_requiredFeatures;
    SVGSynchronizableAnimatedProperty<SVGStringList> m_requiredExtensions;
    SVGSynchronizableAnimatedProperty<SVGStringList> m_systemLanguage;
};

} // namespace WebCore

#endif // ENABLE(SVG)
#endif // SVGTests_h
