/*
 * Copyright (C) 2006, 2007, 2009 Apple Inc. All rights reserved.
 * Copyright (C) 2006 Samuel Weinig <sam.weinig@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

module html {

    interface [
        GenerateNativeConverter,
        CustomPushEventHandlerScope
    ] HTMLElement : Element {
                 // iht.com relies on id returning the empty string when no id is present. 
                 // Other browsers do this as well. So we don't convert null to JS null.
                 attribute [Reflect] DOMString id;
                 attribute [Reflect] DOMString title;
                 attribute [Reflect] DOMString lang;
                 attribute [Reflect] DOMString dir;
                 attribute [Reflect=class] DOMString className;
                 readonly attribute DOMTokenList classList;

                 attribute long              tabIndex;
                 attribute boolean           draggable;
                 attribute [Reflect] boolean hidden;

        // Extensions
                 attribute [ConvertNullToNullString] DOMString innerHTML
                     setter raises(DOMException);
                 attribute [ConvertNullToNullString] DOMString innerText
                     setter raises(DOMException);
                 attribute [ConvertNullToNullString] DOMString outerHTML
                     setter raises(DOMException);
                 attribute [ConvertNullToNullString] DOMString outerText
                     setter raises(DOMException);

        Element insertAdjacentElement(in DOMString where,
                                      in Element element)
            raises(DOMException);
        void insertAdjacentHTML(in DOMString where,
                                in DOMString html)
            raises(DOMException);
        void insertAdjacentText(in DOMString where,
                                in DOMString text)
            raises(DOMException);

        readonly attribute HTMLCollection children;

                 attribute [ConvertNullToNullString] DOMString contentEditable
                     setter raises(DOMException);
        readonly attribute boolean isContentEditable;

                 attribute boolean spellcheck;

#if defined(LANGUAGE_OBJECTIVE_C) && LANGUAGE_OBJECTIVE_C
        readonly attribute DOMString titleDisplayString;
#endif
    };

}
